#include <afxwin.h>
#include <math.h>
#include "TriWindow.h"
#include "TriDialog.h"

short * board;
int width;
int current;

TriWindow::TriWindow() {
	CRect rect;
	rect.bottom = 512+32; rect.right=512+13;
	rect.top = 0; rect.left = 0;
	Create(NULL, "Triominos",WS_OVERLAPPEDWINDOW,rect);

	TriDialog td;
	if(td.DoModal()==-1) exit(0);

	k = td.k;
	chosen_x = td.x - 1 ;
	chosen_y = td.y - 1 ;
	
	width = (int) pow(2,k);
	
    board = new short[width*width];
    for(int i = 0; i<width*width; i++) board[i] = 0;
	board[chosen_y*width+chosen_x] = 1;
	current = 2;

	triomino(0,0,width,chosen_x,chosen_y);

	if(k>=9) {
		MessageBox("You have chosen k>=9, which generates a very large board--so large that detail will be obscured int the graphics, but never fear! The board has been dumped to board.txt",
				   "Oh no!");		
	}

	dumpBoard();
}

TriWindow::~TriWindow() {
	delete board;
}

afx_msg void TriWindow::OnPaint() {
	CPaintDC dc(this);
	
	CRect rect;
	GetClientRect(rect);

	// this portion draws the multi-colored triominos

	// color the boxes
	char text[2];
	CRect box;
	box.top = 1; box.left = 1;
	box.bottom = 512 / width; box.right = 512 / width;
	for(int y = 0; y<width; y++) {
		
		box.top = y * (512/width) + 1;
		box.bottom = box.top + 512/width;

		for(int x=0; x<width; x++) {
			
			box.left = x * (512/width) + 1;
			box.right = box.left + 512/width;
			
			if(board[y*width+x]==1) { // make brush black
				dc.FillSolidRect(box,RGB(0,0,0));
				
			} else { // make brush a "random" color
				dc.FillSolidRect(box,RGB((board[y*width+x]*37) % 256,
										 (board[y*width+x]*17) % 256,
										 (board[y*width+x]*89) % 256));
			}

			
	//		MessageBox("");
			if(k<5) { // put the numbers in the boxes
				text[0]=text[1]=0;
				itoa(board[y*width+x],&text[0],10);
				if(board[y*width+x]>9)
					dc.TextOut((box.left+256/width)-5,box.top+256/width-6,text,2);
				else
					dc.TextOut((box.left+256/width)-5,box.top+256/width-6,text,1);
			}
			
		}

		
	}

	

}

void TriWindow::dumpBoard() {
	FILE * outfile;
	outfile = fopen("board.txt","w");
	if(outfile==NULL) {
		MessageBox("Couldn't open file for writing (board.txt)",
				   "FATAL ERROR!", MB_ICONERROR);
		exit(-1);
	}

	// attempts to pretty-print the table to a large text file
	// won't work too well when the number of triominos exceeds
	// 999
	int space_count;

	for(int y = 0; y<width; y++) {

		for(int x=0; x<width; x++) {
			
			space_count = 4-fprintf(outfile,"%i ",board[y*width+x]);
			for(int i=0; i<space_count; i++) fprintf(outfile," ");
			
		}
		
		fprintf(outfile,"\n\n\n");
	}

	fclose(outfile);
}

void TriWindow::triomino(int start_x, int start_y, int size, int used_x, int used_y) {
   if(size==2) { // base case
      // here must place a triomino to cover 3 uncovered squares

      if(board[start_y*width+start_x] == 0)
         board[start_y*width+start_x] = current;

      if(board[start_y*width+start_x+1] == 0)
         board[start_y*width+start_x+1] = current;

      if(board[(start_y+1)*width+start_x] == 0)
         board[(start_y+1)*width+start_x] = current;

      if(board[(start_y+1)*width+start_x+1] == 0)
         board[(start_y+1)*width+start_x+1] = current;

      current++;

   } else { // recursive step

      // place a triomino such that all quadrants will have 1 filled
      // square

      // if used square is in 1st quadrant
      if( used_y < start_y+size/2 && used_x >= start_x+size/2 ) {
         // place a triomino cover 2-4 quadrants

         board[(start_y+size/2)*width+start_x+size/2] = current;     // 2
         board[(start_y+size/2)*width+start_x+size/2-1] = current;   // 3
         board[(start_y+size/2-1)*width+start_x+size/2-1] = current; // 4

         current++;

         // call triomino for each of the quadrants
         // Quadrant 1:
         triomino(start_x+size/2, start_y, size/2,
                  used_x, used_y);

         // Quadrant 2:
         triomino(start_x+size/2, start_y+size/2, size/2,
                  start_x+size/2, start_y+size/2);

         // Quadrant 3:
         triomino(start_x, start_y+size/2, size/2,
                  start_x+size/2-1, start_y+size/2);

         // Quadrant 4:
         triomino(start_x, start_y, size/2,
                   start_x+size/2-1, start_y+size/2-1);

      }

      // if used square is in 2nd quadrant
      else
      if( used_y >= start_y+size/2 && used_x >= start_x+size/2 ) {
         // place a triomino cover 1,3,4 quadrants

         board[(start_y+size/2-1)*width + start_x+size/2] = current; // 1
         board[(start_y+size/2)*width+start_x+size/2-1] = current;   // 3
         board[(start_y+size/2-1)*width+start_x+size/2-1] = current; // 4

         current++;

         // call triomino for each of the quadrants
         // Quadrant 1:
         triomino(start_x+size/2, start_y, size/2,
                  start_x+size/2, start_y+size/2-1);

         // Quadrant 2:
         triomino(start_x+size/2, start_y+size/2, size/2,
                  used_x, used_y);

         // Quadrant 3:
         triomino(start_x, start_y+size/2, size/2,
                  start_x+size/2-1, start_y+size/2);

         // Quadrant 4:
         triomino(start_x, start_y, size/2,
                   start_x+size/2-1, start_y+size/2-1);


      }

      // if used square is in 3rd quadrant
      else
      if( used_y >= start_y+size/2 ) {
         // place a triomino cover 1,2,4 quadrants

         board[(start_y+size/2-1)*width + start_x+size/2] = current; // 1
         board[(start_y+size/2)*width+start_x+size/2] = current;     // 2
         board[(start_y+size/2-1)*width+start_x+size/2-1] = current; // 4

         current++;

         // call triomino for each of the quadrants
         // Quadrant 1:
         triomino(start_x+size/2, start_y, size/2,
                  start_x+size/2, start_y+size/2-1);

         // Quadrant 2:
         triomino(start_x+size/2, start_y+size/2, size/2,
                  start_x+size/2, start_y+size/2);

         // Quadrant 3:
         triomino(start_x, start_y+size/2, size/2,
                  used_x, used_y);

         // Quadrant 4:
         triomino(start_x, start_y, size/2,
                   start_x+size/2-1, start_y+size/2-1);


      }

      // else used square is in 4th quadrant
      else {
         // place a triomino cover 1-3 quadrants
         board[(start_y+size/2-1)*width + start_x+size/2] = current; // 1
         board[(start_y+size/2)*width+start_x+size/2] = current;     // 2
         board[(start_y+size/2)*width+start_x+size/2-1] = current;   // 3 

         current++;

         // call triomino for each of the quadrants
         // Quadrant 1:
         triomino(start_x+size/2, start_y, size/2,
                  start_x+size/2, start_y+size/2-1);

         // Quadrant 2:
         triomino(start_x+size/2, start_y+size/2, size/2,
                  start_x+size/2, start_y+size/2);

         // Quadrant 3:
         triomino(start_x, start_y+size/2, size/2,
                  start_x+size/2-1, start_y+size/2);

         // Quadrant 4:
         triomino(start_x, start_y, size/2,
                   used_x, used_y);

      }

   }
}

BEGIN_MESSAGE_MAP( TriWindow, CFrameWnd )
	ON_WM_PAINT()
END_MESSAGE_MAP()

class TriominosApp : public CWinApp {
public:
	BOOL InitInstance() {
		m_pMainWnd = new TriWindow;
		m_pMainWnd->ShowWindow( m_nCmdShow );
		m_pMainWnd->UpdateWindow();
		return TRUE;
	}
} triominosapp;